#include "kvaiodriverinterface.h"

extern "C" {
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/sonypi.h>
#include <X11/Xlib.h>
#include <X11/extensions/XTest.h>
}

#include <qsocketnotifier.h>
#include <qtimer.h>
#include <kdebug.h>

// Taken from Stelian Pop's spicctrl utility:

/* Have our own definition of ioctls... */

/* get/set brightness */
#define SONYPI_IOCGBRT		_IOR('v', 0, __u8)
#define SONYPI_IOCSBRT		_IOW('v', 0, __u8)

/* get battery full capacity/remaining capacity */
#define SONYPI_IOCGBAT1CAP	_IOR('v', 2, __u16)
#define SONYPI_IOCGBAT1REM	_IOR('v', 3, __u16)
#define SONYPI_IOCGBAT2CAP	_IOR('v', 4, __u16)
#define SONYPI_IOCGBAT2REM	_IOR('v', 5, __u16)

/* get battery flags: battery1/battery2/ac adapter present */
#define SONYPI_BFLAGS_B1	0x01
#define SONYPI_BFLAGS_B2	0x02
#define SONYPI_BFLAGS_AC	0x04
#define SONYPI_IOCGBATFLAGS	_IOR('v', 7, __u8)

/* get/set bluetooth subsystem state on/off */
#define SONYPI_IOCGBLUE         _IOR('v', 8, __u8)
#define SONYPI_IOCSBLUE         _IOW('v', 9, __u8)

KVaioDriverInterface::KVaioDriverInterface(QObject *parent)
    : QObject(parent),
      mFd(0),
      mNotifier(0)
{
    mTimer = new QTimer(this);
    connect(mTimer, SIGNAL(timeout()),
            SLOT(timeout()));
    mTimer->start(100, true);
}

bool KVaioDriverInterface::connectToDriver()
{
    const char* DriverFile = "/dev/sonypi";

    mFd = open(DriverFile, O_RDONLY);

    // mFd = open(DriverFile, O_RDWR);

    if(mFd == -1)
    {
        kdDebug() << "KVaio: Failed to open /dev/sonypi: "
                  << strerror(errno) << "." << endl;
        return false;
    }

    fcntl(mFd, F_SETFL, fcntl(mFd, F_GETFL) | O_ASYNC);

    mNotifier = new QSocketNotifier(mFd,  QSocketNotifier::Read, this);
    connect(mNotifier, SIGNAL(activated(int)), SLOT(socketActivated(int)));
    return true;
}

void KVaioDriverInterface::disconnectFromDriver()
{
    if(mNotifier!=0)
    {
        delete mNotifier; mNotifier = 0;
    }

    if(mFd!=0)
    {
        close(mFd);
        mFd = 0;
    }
}

void KVaioDriverInterface::socketActivated(int)
{
    unsigned char events[8];
    int count;

    do {
        count = read(mFd, &events, sizeof(events));
        for(int index = 0; index<count; index++)
        {
            emit(vaioEvent(events[index]));
        }
    } while(count==sizeof(events));
}

int KVaioDriverInterface::brightness()
{
    static unsigned char cache; // store last result in here
    unsigned char value = 0;
    int result=0;

    result = ioctl(mFd, SONYPI_IOCGBRT, &value);


//     kdDebug() << "KVaioDriverInterface::brightness: " << (int)value << endl;

    if (result >= 0)
    {
        if(value!=cache)
        {
            emit(brightnessChanged(value));
            cache = value;
        }
        return value;
    } else {
        return -1;
    }

}

void KVaioDriverInterface::setBrightness(int value)
{
    static unsigned char cache; // to avoid unnecessary updates
    int result;
    unsigned char value8 = 0;

    if(value<0) value=0;
    if(value>255) value=255;

    value8 = __u8(value);

    if(value8 != cache)
    {
        result = ioctl(mFd, SONYPI_IOCSBRT, &value8);

        if(result<0)
        {
            kdDebug() << "KVaioDriverInterface::setBrightness: ioctl failed."
                      << endl;
        }
        cache = value8;
        emit(brightnessChanged(value8));
    }
}

void KVaioDriverInterface::timeout()
{
    brightness(); // query and signal on change
    getBatteryStatus();
}

bool KVaioDriverInterface::getBatteryStatus()
{
    static bool init = false;

    bool bat1Avail, bat2Avail, acConnected;

    int bat1Cap, bat2Cap, bat1Rem, bat2Rem;

    unsigned char batFlags = 0;

    bool retval  = true;

    if(ioctl(mFd, SONYPI_IOCGBATFLAGS, &batFlags)<0)
    {
        retval = false;
    } else {
        __u16 rem1 = 0, rem2 = 0, cap1 = 0, cap2 = 0;

        bat1Avail = batFlags & SONYPI_BFLAGS_B1;
        bat2Avail = batFlags & SONYPI_BFLAGS_B2;
        acConnected = batFlags & SONYPI_BFLAGS_AC;

//         kdDebug() << "KVaioDriverInterface::getBatteryStatus: battery 1:  "
//                   << (bat1Avail ? "available" : "not available") << endl
//                   << "                                        battery 2:  "
//                   << (bat2Avail ? "available" : "not available") << endl
//                   << "                                        AC adapter: "
//                   << (acConnected ? "connected" : "connected")
//                   << "." << endl;

        if(bat1Avail
           && ioctl(mFd, SONYPI_IOCGBAT1CAP, &cap1) >= 0
           && ioctl(mFd, SONYPI_IOCGBAT1REM, &rem1) >= 0)
        {
            bat1Cap = cap1;
            bat1Rem = rem1;
        } else {
            bat1Rem = 0;
            bat1Cap = 0;
            retval = false;
        }

        if(bat2Avail
           && ioctl(mFd, SONYPI_IOCGBAT2CAP, &cap2) >= 0
           && ioctl(mFd, SONYPI_IOCGBAT2REM, &rem2) >= 0)
        {
            bat2Cap = cap2;
            bat2Rem = rem2;
        } else {
            bat2Rem = 0;
            bat2Cap = 0;
            retval = false;
        }

        emit(batteryStatus(bat1Avail, bat1Rem, bat1Cap,
                           bat2Avail, bat2Rem, bat2Cap,
                           acConnected));
    }

    if(init == false)
    {
        mTimer->start(2345);
        init = true;
    }

    return retval;
}

#include "kvaiodriverinterface.moc"

